#include "headers.h"

void Assemble(int dof_loc, int x_int_loc, int y_int_loc,
              double x0, double y0, int x_subdoms, int y_subdoms,
              double hx, double hy,
              int *RowPtr, int *KCol, double *matrix_entries,
              double *rhs)
{
  int i, n, ii, jj, kk, ll, ind[3], index, nx, start, end;
  double val, x[3], y[3], xs, ys, area;
  double vx_ii, vy_ii, vx_jj, vy_jj;
  
  nx = x_int_loc+1;
  /* loop over all rectangles containing two triangles */
  n = 2*x_int_loc * y_int_loc;
  /* area of the triangles */
  area = hx*hy/2;   
  index = -2;
  for (i=0;i<n; i++)
  {
    /* first triangel, left lower */
    if ((i%2)==0)
    {
      index++;
      if (((i/2) % x_int_loc)== 0)
        index++;
      /*printf("%d \n",index);*/
      ind[0] = index;
      ind[1] = index+nx;
      ind[2] = index+nx+1;
    }
    else
    /* second triangel, right upper */
    {
      ind[0] = index;
      ind[2] = index+nx+1;
      ind[1] = index+1;
    }
    
    /******************************************************************/
    /* assemble right hand side                                       */
    /******************************************************************/
    /* compute barycentre of the triangle */
    xs = ys = 0;
    for (ii=0;ii<3;ii++)
    {
      ComputeCoordinates(x0, y0, ind[ii], x_subdoms, y_subdoms,  
                         x_int_loc, y_int_loc, x+ii, y+ii);
      xs += x[ii];
      ys += y[ii];
    }
    xs /= 3;
    ys /= 3;
    val = RightHandSide(xs,ys);

    /* accumulate right hand side */
    for (ii=0;ii<3;ii++)
      rhs[ind[ii]] += val * area/3.0;
      
    /******************************************************************/
    /* assemble matrix                                                */
    /******************************************************************/
    
    for (ii=0;ii<3;ii++)
    {
      // correspondig row in matrix structure 
      start = RowPtr[ind[ii]];      
      end = RowPtr[ind[ii]+1];      
 
      /* compute gradient of basis function ii */
      jj = (ii+1)%3;
      kk = (ii+2)%3;
      vx_ii = y[kk]-y[jj];
      vy_ii = x[jj]-x[kk];
   
      for (jj=0;jj<3;jj++)
      {
        /* compute gradient of basis function jj */
        ll = (jj+1)%3;
        kk = (jj+2)%3;
        vx_jj = y[kk]-y[ll];
        vy_jj = x[ll]-x[kk];

        /* compute update for matrix entry */
        val = vx_ii * vx_jj + vy_ii * vy_jj;
        val /= (4*area);

        /* update matrix entry */
        for (kk=start;kk<end;kk++)
        {
          if ((KCol[kk])==ind[jj])
          {           
            matrix_entries [kk] += val;
            break;
          }
        }
      } /* end jj */
    } /* end ii */    
  }

  return;
}
